<?php

namespace Teknisa\TCO\Authentication;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class AuthFunctions{

    protected static $connection;
    protected static $environment;
    protected static $dataSourceManager;

    public function __construct($conn, $env, $dtSrcMgr){
        self::$connection        = $conn;
        self::$environment       = $env;
        self::$dataSourceManager = $dtSrcMgr;
    }

    /**
     * User-Made functions.
     * The implemented functions should return
     * a boolean result and have to be implemented
     * as a public static function
     */


    /**
     * Check if a user is a valid planning
     * operator.
     * 
     * @return bool True if is user defined as
     *  planning user.
     */
    public static function validatePlanningUser(){
        $currentUser = self::$environment->getCdOperador();
        
        $sqlString = "SELECT
                OPERADOR.CDOPERADOR,
                OPERADOR.NMOPERADOR,
                PERFACESSPARC.*
            FROM PARAMTCO PT
                CROSS JOIN PERFACESSPARC
                JOIN OPERADOR ON ( PERFACESSPARC.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO
                                AND PERFACESSPARC.NRORG = OPERADOR.NRORG )
                WHERE PERFACESSPARC.NRPERFILACESSO = PT.NRPERFILACESSO
                AND OPERADOR.CDOPERADOR = :CDOPERADOR";

        $preparedQuery = self::$connection->prepare($sqlString);
        $params = array(
            "CDOPERADOR"=>$currentUser
        ); 

        $preparedQuery->execute($params);
        $result = $preparedQuery->fetchAll();
        
        return (bool)sizeof($result);
    }

    /**
     * Valida ações do gerente de negócio na tela
     * de serviço proposta comercial de acordo com o estado
     * do serviço da proposta;
     * 
     * @param object $request
     */
    public static function validateGNActionOnServPropCom($request){
        $calledRoute = $request->getRoutePath();
        $row = ($request instanceof DTO\Request\DataSet) ? $request->getDataSet()->getRows()[0] : $request->getRow();

        $conditions = [
            ["columnName" => "NRSERVICOPROPCOM", "operator" => "=","value" => $row->get("NRSERVICOPROPCOM")]
        ];

        $filterCriteria = new FilterCriteria("servicoPropostaComercial", $conditions);
        $dataSet = self::$dataSourceManager->findBy($filterCriteria)->getRows();

        if($dataSet && sizeof($dataSet)){
            $allowedActionsByRoute = [
                "/tco01101DetalhesServico/saveEditServicoProposta"  => ['PENDENTE', 'CALCULADO', 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/calcularCustServ"         => ["PENDENTE", 'CALCULADO', 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/saveEditTipoPrat"         => ["PENDENTE", 'CALCULADO', 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/saveAddPorcGram"          => ["PENDENTE", 'CALCULADO', 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/confirmCustomize"         => ["PENDENTE", 'CALCULADO', 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/notificarPlan"            => ["PENDENTE", 'CALCULADO', 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/selectMarca"              => ["CALCULADO", 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/clearMarcaProduto"        => ["CALCULADO", 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/saveIncidencias"          => ["CALCULADO", 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/openServCompl"            => ["CALCULADO", 'FINALIZADOPLANEJAMENTO', 'RECUSADO'],
                "/tco01101DetalhesServico/enviaResposta"            => ["AGUARDANDOGN"],
            ];

            return in_array($dataSet[0]['IDSERVICOPROPCOM'], $allowedActionsByRoute[$calledRoute]);
        } else return false; 
    }

    /**
     * Valida versão da proposta ao tentar alterar
     * dados do serviço da proposta.
     * 
     * @param object $request
     */
    public static function validateIdVersaoPropCom($request){
        $calledRoute = $request->getRoutePath();
        $row = ($request instanceof DTO\Request\DataSet) ? $request->getDataSet()->getRows()[0] : $request->getRow();

        $conditions = [
            ["columnName" => "NRVERSAOPROPCOM", "operator" => "=","value" => $row->get("NRVERSAOPROPCOM")]
        ];

        $filterCriteria = new FilterCriteria("versaopropcom", $conditions);
        $dataSet = self::$dataSourceManager->findBy($filterCriteria)->getRows();

        return ($dataSet && sizeof($dataSet) &&
             $dataSet[0]['IDVERSAOPROPCOM']=='PENDENTE');
    }
}