<?php
namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Tco01006RegrasCardapioService extends \Teknisa\GER\Controllers\TeknisaCrud{
    
    protected $newCode;
    protected $tcoList;
    protected $connection;
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager, $connection, $dataSourceManager, $newCode, $tcoList) {
        $this->newCode              = $newCode;
        $this->tcoList              = $tcoList;
        $this->connection           = $connection;
        $this->crudService          = $crudService;
        $this->environment          = $environment;
        $this->entityManager        = $entityManager;
        $this->dataSourceManager    = $dataSourceManager;
    }
    
    //retorna o sequencial do código de adesão do cardapio base para um cardapio base 
    public function retornaProxCodAdesaoCard($nrcardapiobase){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT MAX(LPAD(CDADESAOCARDBASE, 5, '0')) CDADESAOCARDBASE
                  FROM ADESAOCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE 
                   AND NRORG = :NRORG";
                   
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE" => $nrcardapiobase,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        if ($result["CDADESAOCARDBASE"]) {
            $result["CDADESAOCARDBASE"] = str_pad($result["CDADESAOCARDBASE"] + 1, 5, "0", STR_PAD_LEFT);
        } else {
            $result["CDADESAOCARDBASE"] = "00001";
        }
        return $result['CDADESAOCARDBASE'];
    }
    
    //Apaga uma regra de adesão do cardapio base
    public function deletaRegraAdesao($nradesaocardbase){
        $sql = "DELETE FROM ATIVCLIADESAOCBASE WHERE NRADESAOCARDBASE = :NRADESAOCARDBASE AND NRORG = :NRORG";
        $params = array(
            "NRADESAOCARDBASE"  => $nradesaocardbase,
            "NRORG"             => $this->environment->getNrOrgTrab()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    	
    	$sql = "DELETE FROM SEGMERADESAOCBASE WHERE NRADESAOCARDBASE = :NRADESAOCARDBASE AND NRORG = :NRORG";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
    //Apaga tipos de prato relacionados a uma regra de adesao do cardapio base
    public function deletaTipoPratoRegraAdesao($nradesaocardbase){
        $sql = "DELETE FROM ITEMADESAOCBASE WHERE NRADESAOCARDBASE = :NRADESAOCARDBASE";
        $params = array(
            "NRADESAOCARDBASE"  => $nradesaocardbase
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
    //Verifica uso da gramatura em propostas comerciais
    public function verificaUsoPropostaComercial($nrgramaturacardbase){
        $sql = "SELECT DISTINCT P.CDPROPOSTACOMERCIAL, V.CDVERSAOPROPCOM
                  FROM CONFGRUPOTPPROPCOM C
                       INNER JOIN DETALHESERVPROPCOM D ON D.NRDETALHESERVPROPCOM = C.NRDETALHESERVPROPCOM
                       LEFT JOIN SERVICOPROPCOM S ON S.NRSERVICOPROPCOM = D.NRSERVICOPROPCOM
                       LEFT JOIN VERSAOPROPCOM V ON V.NRVERSAOPROPCOM = S.NRVERSAOPROPCOM
                       LEFT JOIN PROPOSTACOMERCIAL P ON P.NRPROPOSTACOMERCIAL = V.NRPROPOSTACOMERCIAL
                 WHERE C.NRGRAMATURACARDBASE = :P_NRGRAMATURACARDBASE
                 ORDER BY 1,2";
        $params = array(
            "P_NRGRAMATURACARDBASE"  => $nrgramaturacardbase
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$result = $query->fetchAll(\PDO::FETCH_ASSOC);
    	return $result;
    }
    
    //Apaga os agrupamentos relacionados à regra de gramatura
    public function deletaAgrupamentoRegraGramatura($nrgramaturacardbase){
        $sql = "DELETE FROM ITEMGRAMCARDBASE WHERE NRGRAMATURACARDBASE = :NRGRAMATURACARDBASE";
        $params = array(
            "NRGRAMATURACARDBASE"  => $nrgramaturacardbase
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
    public function retornaCamposDinamicosGramatura($inString, $nrgramaturacardbase){
        $sql = "SELECT * 
                  FROM ( SELECT PORCIONAMENTO.NRPORCIONAMENTO,
                                ITEMGRAMCARDBASE.NRGRUPOPRATO,
                                GRUPOPRATO.CDGRUPOPRATO,
                                GRUPOPRATO.NMGRUPOPRATO,
                                ITEMGRAMCARDBASE.NRTIPOPREPAROTCO,
                                TIPOPREPAROTCO.NMTIPOPREPAROTCO,
                                ITEMGRAMCARDBASE.CDPRODUTO,
                                PRODUTO.NMPRODUTO,
                                ITEMGRAMCARDBASE.VRGRAMATURA,
                                PORCIONAMENTO.NRORG,
                                ITEMGRAMCARDBASE.IDATIVO
                           FROM PORCIONAMENTO 
                          INNER JOIN ITEMGRAMCARDBASE
                             ON ITEMGRAMCARDBASE.NRPORCIONAMENTO = PORCIONAMENTO.NRPORCIONAMENTO
                           LEFT JOIN GRUPOPRATO
                             ON GRUPOPRATO.NRGRUPOPRATO = ITEMGRAMCARDBASE.NRGRUPOPRATO 
                           LEFT JOIN TIPOPREPAROTCO
                             ON TIPOPREPAROTCO.NRTIPOPREPAROTCO = ITEMGRAMCARDBASE.NRTIPOPREPAROTCO
                           LEFT JOIN PRODUTO
                             ON PRODUTO.CDPRODUTO = ITEMGRAMCARDBASE.CDPRODUTO
                          WHERE NRGRAMATURACARDBASE = :NRGRAMATURACARDBASE ) 
                PIVOT (
                   SUM(VRGRAMATURA)
                   FOR NRPORCIONAMENTO IN (".$inString."))";
        $query = $this->connection->prepare($sql);
    	
    	$params = array(
            "NRGRAMATURACARDBASE" => $nrgramaturacardbase
        );
        
    	$query->execute($params);
    	$result = $query->fetchAll(\PDO::FETCH_ASSOC);
    	return $result;
    }
    
    public function retornaProxCodPartic($nrcardapiobase){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT MAX(LPAD(CDPARTSEGCARDBASE, 5, '0')) CDPARTSEGCARDBASE
                  FROM PARTSEGCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE 
                   AND NRORG = :NRORG";
                   
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE" => $nrcardapiobase,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        if ($result["CDPARTSEGCARDBASE"]) {
            $result["CDPARTSEGCARDBASE"] = str_pad($result["CDPARTSEGCARDBASE"] + 1, 5, "0", STR_PAD_LEFT);
        } else {
            $result["CDPARTSEGCARDBASE"] = "00001";
        }
        return $result['CDPARTSEGCARDBASE'];
    }
    
    //Apaga tipos de prato relacionados a uma regra de Particularidade do cardapio base
    public function deletaTipoPratoRegraParticularidade($nradesaocardbase){
        $sql = "DELETE FROM ITPARTSEGCARDBASE WHERE NRPARTSEGCARDBASE = :NRPARTSEGCARDBASE AND NRORG = :NRORG";
        $params = array(
            "NRPARTSEGCARDBASE"  => $nradesaocardbase,
            "NRORG"              => $this->environment->getNrOrgTrab()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    /**
     * Preenche os dados da aba de Descartáveis.
     * Busca os campos de Amostra Quente e Frio.
     * 
     * @param {String|Int} $nrCardapioBase Número do
     *  cardápio base.
     * 
     * @return {Array} Registros de amostra da aba
     *  de descartaveis.
     */
    public function getAmostraData($nrCardapioBase){
        $sql = 'SELECT * FROM AMOSTRACARDBASE WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE';

        $params = array(
            "NRCARDAPIOBASE" => $nrCardapioBase
        );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        $result = sizeof($result) ? $result[0] : false;
        
        $configCardBase = $this->getConfigCardBase(array(
            $result['NRCONFIGCARDBASEF'],
            $result['NRCONFIGCARDBASEQ']
        ));
        
        $dataToReturn = array(
            'NRCONFIGCARDBASEF' => $configCardBase ? array_filter($configCardBase, function($reg) use ($result){
                return $reg['NRCONFIGCARDBASE']==$result['NRCONFIGCARDBASEF'];
            }) : [],
            'NRCONFIGCARDBASEQ' => $configCardBase ? array_filter($configCardBase, function($reg) use ($result){
                return $reg['NRCONFIGCARDBASE']==$result['NRCONFIGCARDBASEQ'];
            }) : []
        );
        
        $dataToReturn = array_map('array_values', $dataToReturn);

        return $dataToReturn;
    }

    /**
     * Retorna os descartáveis de um cardápio base
     * excluindo os de amostra.
     * 
     * @param {String|Int} $nrCardapioBase Número do
     *  cardápio base.
     * 
     * @return {Array} Registros de amostra da aba
     *  de descartaveis.
     */
    public function getDescartaveisCardBase($nrCardapioBase){
        $conditions = array(
            array(
                "columnName" => "NRCARDAPIOBASE", 
                "operator"   => "=",
                "value"      => $nrCardapioBase
            )
        );
        
        $filterCriteria = new FilterCriteria("tco01006_descartaveis", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();

        return sizeof($dataSet) ? $dataSet : false;
    }
    
    /**
     * Retorna tipo de prato com nome e código do tipo
     * de acordo com a configuração do cardapio base.
     * 
     * @param {String} $nrCardapioBase Numero do 
     *  cardapio base.
     * 
     * @return {Array} Registros de tipo de prato.
     */
    public function getConfigCardBase($nrConfigCardBase){
        $conditions = array(
            array(
                "columnName" => "NRCONFIGCARDBASE", 
                "operator"   => "IN",
                "value"      => $nrConfigCardBase
            )
        );
    
        $filterCriteria = new FilterCriteria("configcardbase_tipoprat", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
        
        return sizeof($dataSet) ? $dataSet : false;
    }

    /**
     * Verifica se as amostras de descartaveis
     * são descartaveis válidos do cardapio.
     */
    public function isValidAmostraFromCardBase($row){
        $isValid = true;
        $nrConfigCardBase = array(
            $row['NRCONFIGCARDBASEF'],
            $row['NRCONFIGCARDBASEQ']
        );

        if($nrConfigCardBase[0]!=$nrConfigCardBase[1]){
            $conditions = array(
                array(
                    "columnName" => "NRCONFIGCARDBASE", 
                    "operator"   => "IN",
                    "value"      => $nrConfigCardBase
                ),
                array(
                    "columnName" => "IDCLASSIFICACAO", 
                    "operator"   => "=",
                    "value"      => "DESCARTAVEL"
                ),
                array(
                    "columnName" => "NRCARDAPIOBASE", 
                    "operator"   => "=",
                    "value"      => $row['NRCARDAPIOBASE']
                )
            );
            $filterCriteria = new FilterCriteria("configcardbase_tipoprat", $conditions);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
            $dataSet = array_map(function($reg){
                return $reg['NRCONFIGCARDBASE'];
            }, $dataSet);

            foreach($nrConfigCardBase as $reg){
                if($reg && !in_array($reg, $dataSet)){
                    $isValid = false;
                }
            }
        } else if($nrConfigCardBase[0]) {
            $isValid = false;
        }
    
        return $isValid;
    }

    /**
     * Salva alteração ou inserção em uma amostra
     * de descartáveis de um cardapio base.
     * 
     * @param {Array} $row Row contendo dados
     *  para operação.
     */
    public function saveAmostra($row){
        $nrOrg = $this->environment->getNrOrg();
        $nrAmostraCardBase = $this->newCode->getCodeByTableName('AMOSTRACARDBASE', $nrOrg, 1, 10, true);

        $sql = "MERGE INTO AMOSTRACARDBASE AMCB
            USING (SELECT :NRCARDAPIOBASE NRCARDAPIOBASE
                    FROM DUAL ) SRC
            ON (SRC.NRCARDAPIOBASE = AMCB.NRCARDAPIOBASE)
            WHEN MATCHED THEN 
                UPDATE
                    SET AMCB.NRCONFIGCARDBASEQ  = :NRCONFIGCARDBASEQ,
                    AMCB.NRCONFIGCARDBASEF      = :NRCONFIGCARDBASEF,
                    AMCB.DTULTATU               = :DTULTATU,
                    AMCB.NRORGULTATU            = :NRORGULTATU,
                    AMCB.CDOPERULTATU           = :CDOPERULTATU
                WHERE
                    AMCB.NRCARDAPIOBASE = :NRCARDAPIOBASE
            WHEN NOT MATCHED THEN 
                INSERT
                    (NRAMOSTRACARDBASE, NRCARDAPIOBASE, NRCONFIGCARDBASEQ, NRCONFIGCARDBASEF,
                    DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU)
                VALUES
                    (:NRAMOSTRACARDBASE, :NRCARDAPIOBASE, :NRCONFIGCARDBASEQ, :NRCONFIGCARDBASEF,
                    :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        
        $params = array(
            "NRCARDAPIOBASE"    => $row["NRCARDAPIOBASE"],
            "NRCONFIGCARDBASEQ" => $row["NRCONFIGCARDBASEQ"] ?: null,
            "NRCONFIGCARDBASEF" => $row["NRCONFIGCARDBASEF"] ?: null,
            "NRAMOSTRACARDBASE" => $nrAmostraCardBase,
            "DTINCLUSAO"        => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
            "DTULTATU"          => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
            "NRORGINCLUSAO"     => $nrOrg,
            "NRORGULTATU"       => $nrOrg,
            "CDOPERINCLUSAO"    => $this->environment->getCdOperador(),
            "CDOPERULTATU"      => $this->environment->getCdOperador()
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    /**
     * Altera forma de calculo de descartaveis de
     * um cardapio base;
     * 
     * @param {Array} $row Registros a serem 
     * alterados.
     */
    public function changeFormaCalculo($row){
        $formaCalculo = isset($row['formaCalculo']['IDFORMACALCULO']) ? $row['formaCalculo']['IDFORMACALCULO'] : 'DELETE';
        $rowsToChange = array_map(function($reg){
            return array(
                "NRCARDAPIOBASE"=>$reg["NRCARDAPIOBASE"],
                "NRCONFIGCARDBASE"=>$reg["NRCONFIGCARDBASE"]
            );
        }, $row['selectedRows']);

        if($formaCalculo=='DELETE'){
            $this->deleteDescartCardBase($rowsToChange);
        } else {
            foreach ($rowsToChange as $key => $row) {
                $this->mergeDescartCardBase($formaCalculo, $row);
            }
        }
    }

    /**
     * Salva alteração ou inserção de uma forma de calculo
     * em um tipo de prato de descartáveis de um cardapio base.
     * 
     * @param {Array} $row Row contendo dados
     *  para operação.
     */
    private function mergeDescartCardBase($formaCalculo, $row){
        $nrOrg = $this->environment->getNrOrg();
        $nrDescartCardBase = $this->newCode->getCodeByTableName('DESCARTCARDBASE', $nrOrg, 1, 10, true);

        $sql = "MERGE INTO DESCARTCARDBASE DSCB
                USING (SELECT :NRCARDAPIOBASE NRCARDAPIOBASE,
                            :NRCONFIGCARDBASE NRCONFIGCARDBASE
                        FROM DUAL ) SRC
                ON ((SRC.NRCARDAPIOBASE   = DSCB.NRCARDAPIOBASE) AND (SRC.NRCONFIGCARDBASE = DSCB.NRCONFIGCARDBASE))            
                WHEN MATCHED THEN 
                    UPDATE
                        SET DSCB.IDFORMACALCULO     = :IDFORMACALCULO,
                        DSCB.DTULTATU               = :DTULTATU,
                        DSCB.NRORGULTATU            = :NRORGULTATU,
                        DSCB.CDOPERULTATU           = :CDOPERULTATU
                    WHERE
                        DSCB.NRCARDAPIOBASE   = :NRCARDAPIOBASE
                        AND DSCB.NRCONFIGCARDBASE = :NRCONFIGCARDBASE
                WHEN NOT MATCHED THEN 
                    INSERT
                        (NRDESCARTCARDBASE, NRCONFIGCARDBASE, NRCARDAPIOBASE, IDFORMACALCULO,
                        DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU)
                    VALUES
                        (:NRDESCARTCARDBASE, :NRCONFIGCARDBASE, :NRCARDAPIOBASE, :IDFORMACALCULO,
                        :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        
        $params = array(
            "NRCARDAPIOBASE"    => $row["NRCARDAPIOBASE"],
            "NRCONFIGCARDBASE"  => $row["NRCONFIGCARDBASE"],
            "IDFORMACALCULO"    => $formaCalculo,
            "NRDESCARTCARDBASE" => $nrDescartCardBase,
            "DTINCLUSAO"        => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
            "DTULTATU"          => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
            "NRORGINCLUSAO"     => $nrOrg,
            "NRORGULTATU"       => $nrOrg,
            "CDOPERINCLUSAO"    => $this->environment->getCdOperador(),
            "CDOPERULTATU"      => $this->environment->getCdOperador()
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    /**
     * Deleta registros da tabela de descartcardbase
     * excluindo as formas de calculo de cada 
     * configcardbase.
     * 
     * @param {Array} $rowsToRemove Regsitros a serem
     * apagados.
     */
    public function deleteDescartCardBase($rowsToRemove){
        $sql = "DELETE FROM DESCARTCARDBASE 
            WHERE 
                NRCARDAPIOBASE = :NRCARDAPIOBASE
                AND VALOR_NA_LISTA(NRCONFIGCARDBASE, :NRCONFIGCARDBASE) > 0";

        $nrCardapioBase = $rowsToRemove[0]['NRCARDAPIOBASE'];
        
        $rowsToRemove = array_map(function($reg){
            return $reg['NRCONFIGCARDBASE'];
        }, $rowsToRemove);

        $rowsToRemove = implode('_', $rowsToRemove);
        $rowsToRemove = $this->tcoList->criarListaValores($rowsToRemove);

        $params = array(
            "NRCARDAPIOBASE" => $nrCardapioBase,
            "NRCONFIGCARDBASE" => $rowsToRemove
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        $this->tcoList->apagarListaValores($rowsToRemove);
    }

    /**
     * Salva alteração de registro de tipo de prato
     * sinalizado como limpeza na configuração de 
     * cardapio base.
     * 
     * @param {Array} $rows Array de rows  a serem
     *  salvas.
     * 
     * @return {Array|String} Estado de validade
     *  dos registros a serem salvos.
     */
    public function saveLimpeza($rows){
        $this->entityManager->beginTransaction();

        foreach ($rows as $key => $row) {
            $this->mergeLimpezaCardBase($row);
        }

        $nrcardapiobase = $rows[0]['NRCARDAPIOBASE'];
        $invalidLimpezaRows = $this->getInvalidLimpezaRows($nrcardapiobase);
        if(!$invalidLimpezaRows){
            $this->entityManager->commit();
        } else {
            $this->entityManager->rollback();
        }

        return $invalidLimpezaRows;
    }

    /**
     * Salva alteração ou inserção de quantidades
     * de um tipo de prato de limpeza.
     */
    public function mergeLimpezaCardBase($row){
        $nrOrg = $this->environment->getNrOrg();

        $nrLimpezaCardBase = $row->has('NRLIMPEZACARDBASE') && $row->get('NRLIMPEZACARDBASE') ?
            $row->get('NRLIMPEZACARDBASE') : 
            $this->newCode->getCodeByTableName('LIMPEZACARDBASE', $nrOrg, 1, 10, true);

        $sql = "MERGE INTO LIMPEZACARDBASE LPCB
            USING (SELECT :NRLIMPEZACARDBASE NRLIMPEZACARDBASE
                    FROM DUAL ) SRC
            ON (SRC.NRLIMPEZACARDBASE = LPCB.NRLIMPEZACARDBASE)
            WHEN MATCHED THEN 
                UPDATE
                    SET LPCB.QTINICIAL          = :QTINICIAL,
                    LPCB.QTFINAL                = :QTFINAL,
                    LPCB.DTULTATU               = :DTULTATU,
                    LPCB.NRORGULTATU            = :NRORGULTATU,
                    LPCB.CDOPERULTATU           = :CDOPERULTATU
                WHERE
                    LPCB.NRLIMPEZACARDBASE = :NRLIMPEZACARDBASE
            WHEN NOT MATCHED THEN 
                INSERT
                    (NRLIMPEZACARDBASE, NRCARDAPIOBASE, NRCONFIGCARDBASE, QTINICIAL, QTFINAL,
                    DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU)
                VALUES
                    (:NRLIMPEZACARDBASE, :NRCARDAPIOBASE, :NRCONFIGCARDBASE, :QTINICIAL, :QTFINAL,
                    :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        
        $params = array(
            "NRCARDAPIOBASE"    => $row->get("NRCARDAPIOBASE"),
            "NRCONFIGCARDBASE"  => $row->get("NRCONFIGCARDBASE"),
            "QTINICIAL"         => $row->has("QTINICIAL") ? $row->get("QTINICIAL") : 0,
            "QTFINAL"           => $row->has("QTFINAL") ? $row->get("QTFINAL") : 0,
            "NRLIMPEZACARDBASE" => $nrLimpezaCardBase,
            "DTINCLUSAO"        => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
            "DTULTATU"          => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
            "NRORGINCLUSAO"     => $nrOrg,
            "NRORGULTATU"       => $nrOrg,
            "CDOPERINCLUSAO"    => $this->environment->getCdOperador(),
            "CDOPERULTATU"      => $this->environment->getCdOperador()
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    /**
     * Valida os registros de limpeza a serem salvos
     * e retorna os registros invalidos.
     * 
     * @param {int} $nrcardapiobase Numero do cardapio
     *  dos registros a serem verificados.
     * 
     * @return {Array|Boolean} Retorna o array de registros
     *  invalidos ou 'false' caso não exista registros
     *  invalidos.
     */
    private function getInvalidLimpezaRows($nrcardapiobase){
        $invalidRegisters = array();

        $limpezaData = $this->getLimpezaData($nrcardapiobase);
        $valuesArray = array();

        foreach ($limpezaData as $key => $reg) {
            $isInvalidLimpezaQuant = $this->isInvalidLimpezaQuant($reg);
            if($isInvalidLimpezaQuant){
                $invalidRegisters[] = array(
                    'ERROR_CODE'=>$isInvalidLimpezaQuant,
                    'PARAMS'=>$reg
                );
            }

            $valuesArray[] = (int)$reg['QTINICIAL'];
            $valuesArray[] = (int)$reg['QTFINAL'];
        }
        sort($valuesArray);

        $invalidArrayIntervals = $this->getInvalidIntervalsLimpeza($limpezaData, $valuesArray);
        if($invalidArrayIntervals) {
            $invalidRegisters = array_merge($invalidRegisters, $invalidArrayIntervals);
        }

        return sizeof($invalidRegisters) ? $invalidRegisters : false;
    }

    /**
     * valida os valores de comensais do
     * registro de limpezacardbase entre si.
     * 
     * @param {Array} $row Registro de limpeza.
     * 
     * @param {String|Bool} Retorna o código do
     *  erro ou false caso seja valido.
     */
    private function isInvalidLimpezaQuant($reg){
        $invalid = false;

        if($reg['QTINICIAL']>$reg['QTFINAL']){
            $invalid = 'TCO01006_LIMPEZACARDBASE_BEGIN_MT_END';
        } else if(!$reg['QTINICIAL'] || !$reg['QTFINAL']){
            $invalid = 'TCO01006_LIMPEZACARDBASE_ZERO_REGISTER';
        }

        return $invalid;
    }


    /**
     * Valida os intervalos do array de registros
     * dentro do array de valores, seguindo as regras
     * de limpeza. Os intervalos não podem se interpolar.
     * 
     * @param {array} $regArray Array de registros de 
     *  limpezacardbase a serem verificados.
     * @param {array} $valuesArray Array de valores de
     *  intervalos a serem usados como referência.
     * 
     * @return {array|bool} Retorna um array de registros
     *  com intervalos invalidos ou false caso não haja
     *  invalidez.
     */
    private function getInvalidIntervalsLimpeza($regArray, $valuesArray){
        $invalidIntervalsReg = array();
        $countedValuesArray = array_count_values($valuesArray);

        foreach ($regArray as $regKey => $reg) {
            $qti = $reg['QTINICIAL'];
            $qtf = $reg['QTFINAL'];
            $qtiIndex = array_search($qti, $valuesArray); 
            $qtfIndex = array_search($qtf, $valuesArray);
            
            if(($qti==$qtf && $countedValuesArray[$qti]!=2)||
            ($qti!=$qtf && ($countedValuesArray[$qti]!=1 ||
                $countedValuesArray[$qtf]!=1 ||
                $qtiIndex!=($qtfIndex-1)))){
                
                $invalidIntervalsReg[] = array(
                    'ERROR_CODE'=>'TCO01006_LIMPEZACARDBASE_INVALID_RANGE',
                    'PARAMS'=>$reg
                );
            }
        }

        return sizeof($invalidIntervalsReg) ? $invalidIntervalsReg : false;
    }

    /**
     * Retorna dados de tipos de prato cadastrados
     * como limpeza nas configurações do cardápio.
     * 
     * @param {Int} $nrcardapiobase Numero do 
     *  cardápio base a ser usado como referência.
     */
    private function getLimpezaData($nrcardapiobase){
        $conditions = array(
            array(
                "columnName" => "NRCARDAPIOBASE", 
                "operator"   => "=",
                "value"      => $nrcardapiobase
            )
        );
        
        $filterCriteria = new FilterCriteria("tco01006_limpeza", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
        
        return sizeof($dataSet) ? $dataSet : false;
    }

    /**
     * Remove registros de quantidades de comensais
     * de tipos de prato sinalizados como limpeza.
     * 
     * @param {Array} $rowsToRemove Registros de 
     *  tipo de prato a serem removidos.
     */
    public function removeLimpezaCardBase($rowsToRemove){
        $sql = "DELETE FROM LIMPEZACARDBASE 
            WHERE VALOR_NA_LISTA(NRLIMPEZACARDBASE, :NRLIMPEZACARDBASE) > 0";
        
        $rowsToRemove = implode('_', $rowsToRemove);
        $rowsToRemove = $this->tcoList->criarListaValores($rowsToRemove);

        $params = array(
            "NRLIMPEZACARDBASE" => $rowsToRemove
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        $this->tcoList->apagarListaValores($rowsToRemove);
    }

    /**
     * Retorna os dados de gramatura por tipo
     * de prato de acordo com os parametros
     * passados na row por parametro.
     * 
     * @param {Array} $row valores a serem usados
     *  como referência para filtragem.
     * 
     * @return {Array|Boolean} Retorna o registro
     *  caso seja encontrado ou false caso não 
     *  seja.
     */
    public function getGramatura($row){
        $conditions = array(
            array(
                "columnName" => "NRCARDAPIOBASE", 
                "operator"   => "=",
                "value"      => $row['NRCARDAPIOBASE']
            ),
            array(
                "columnName" => "CDGRTIPOPR", 
                "operator"   => "=",
                "value"      => $row['CDGRTIPOPR']
            ),
            array(
                "columnName" => "VRGRAMATURA", 
                "operator"   => "=",
                "value"      => $row['VRGRAMATURA']
            ),
            array(
                "columnName" => "SGUNIDADE", 
                "operator"   => "=",
                "value"      => $row['SGUNIDADE']
            )
        );
        
        if(!empty($row['NRTIPOPUBLICO'])){
            $conditions[] = array(
                "columnName" => "NRTIPOPUBLICO", 
                "operator"   => "=",
                "value"      => $row['NRTIPOPUBLICO']
            );
        }else{
            $conditions[] = array(
                "columnName" => "NRTIPOPUBLICO", 
                "operator"   => "IS_NULL"
            );
        }
        
        $filterCriteria = new FilterCriteria("gramaturacardbase", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
        return sizeof($dataSet) ? $dataSet : false;
    }
    
    public function existeParticularidadeSemelhante($row){
        $conditions = array(
            array(
                "columnName" => "NRCARDAPIOBASE", 
                "operator"   => "=",
                "value"      => $row['NRCARDAPIOBASE']
            ),
            array(
                "columnName" => "NRATIVCLISEGMER", 
                "operator"   => "=",
                "value"      => $row['NRATIVCLISEGMER']
            )
        );
        
        $filterCriteria = new FilterCriteria("partsegcardbase", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
        
        return sizeof($dataSet) ? $dataSet : false;
    }
    
    public function retornaRamosAdesao($nradesaocardbase){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT S.CDSEGMER
                  FROM SEGMERADESAOCBASE SC
                       INNER JOIN SEGMERCADO S ON S.CDSEGMER = SC.CDSEGMER
                 WHERE SC.NRADESAOCARDBASE = :P_NRADESAOCARDBASE
                 ORDER BY S.DSSEGMER";
                   
        $query = $connection->prepare($sql);
        
        $params = array(
            "P_NRADESAOCARDBASE" => $nradesaocardbase
        );
        
        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        $ramos = array();
        foreach($result as $ramo){
            $ramos[] = $ramo['CDSEGMER'];
        }
        return $ramos;
    }
    
    public function retornaSegmentosAdesao($nradesaocardbase){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT A.NRATIVCLISEGMER
                  FROM ATIVCLIADESAOCBASE A
                       INNER JOIN ATIVCLISEGMER ATS ON ATS.NRATIVCLISEGMER = A.NRATIVCLISEGMER
                       INNER JOIN ATIVCLIENTE AC ON AC.CDATIVCLI = ATS.CDATIVCLI
                 WHERE A.NRADESAOCARDBASE = :P_NRADESAOCARDBASE
                 ORDER BY AC.DSATIVCLI";
                   
        $query = $connection->prepare($sql);
        
        $params = array(
            "P_NRADESAOCARDBASE" => $nradesaocardbase
        );
        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        $segmentos = array();
        foreach($result as $segmento){
            $segmentos[] = $segmento['NRATIVCLISEGMER'];
        }
        return $segmentos;
    }
    
    public function retornaTiposPratoAdesao($nradesaocardbase){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRCONFIGCARDBASE, 
                       NRPORCIONAMENTO, 
                       VRPEADESAO 
                  FROM ITEMADESAOCBASE 
                 WHERE NRADESAOCARDBASE = :P_NRADESAOCARDBASE";
                   
        $query = $connection->prepare($sql);
        
        $params = array(
            "P_NRADESAOCARDBASE" => $nradesaocardbase
        );
        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        for($i = 0; $i< sizeof($result); $i++){
            $result[$i]['__is_new'] = true;
        }
        return $result;
    }
    
    public function retornaColsPorcionamento($cdgrtipopr){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT P.NRPORCIONAMENTO, P.NMPORCIONAMENTO
                  FROM RELPORCIOGRTIPOPR R
                       INNER JOIN PORCIONAMENTO P ON P.NRPORCIONAMENTO = R.NRPORCIONAMENTO
                 WHERE R.CDGRTIPOPR = :CDGRTIPOPR
                 ORDER BY P.CDPORCIONAMENTO";
                   
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDGRTIPOPR" => $cdgrtipopr
        );
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function returnItensGramatura($nrgramaturacardbase) {
        $sql = "SELECT NRITEMGRAMCARDBASE,
                       NRGRAMATURACARDBASE,
                       CDGRUPPROD,
                       CDSUBGRPROD,
                       NRGRUPOPRATO,
                       NRTIPOPREPAROTCO,
                       CDPRODUTO,
                       NRPORCIONAMENTO,
                       VRGRAMATURA,
                       IDATIVO,
                       NRORG
                  FROM ITEMGRAMCARDBASE
                 WHERE NRGRAMATURACARDBASE = :NRGRAMATURACARDBASE
                   AND NRORG = :NRORG";
        $query = $this->connection->prepare($sql);
    	
    	$params = array(
            "NRGRAMATURACARDBASE" => $nrgramaturacardbase,
            "NRORG" => $this->environment->getNrOrgTrab()
        );
        
    	$query->execute($params);
    	$result = $query->fetchAll(\PDO::FETCH_ASSOC);
    	return $result != null? $result: array();
    }
    
    public function saveAddPorcGram($row, $reqParams){
        if(!$row->has('NRCONFGRUPTPCARDBASE') || !$row->get('NRCONFGRUPTPCARDBASE')){
            $nrOrg                = $this->environment->getNrOrg();
            $nrConfGrupoTPCardBase = $this->newCode->getCodeByTableName('CONFGRUPTPCARDBASE', $nrOrg, 1, 10, true);
            $row->set('NRCONFGRUPTPCARDBASE', (float)$nrConfGrupoTPCardBase);
            $row->set('__is_new', true);
        }
        else{
              $row->set('__is_new', false);
        }
        
        $row['NRORG']=$this->environment->getNrOrg();
        $request   = new DTO\Request\Row($row, $reqParams["REQMETHOD"], $reqParams["REQRPATH"], $reqParams["REQUSERID"]);
        $rowToSave = $this->crudService->getRowsToSave($request, 'confgruptpcardbase');
        $dataSet   = new DataSet('confgruptpcardbase', $rowToSave);
        $this->dataSourceManager->persist($dataSet);

     
        
    }
}